{% extends 'allianceauth/base.html' %}
{% load i18n %}
{% load static %}

{% block page_title %}{% trans app_title %}{% endblock %}
{% block content %}
    <div class="col-lg-12">
        <h1 class="page-header text-center">{% trans "Sync of Alliance Standings" %}</h1>
        <div class="col-lg-10 col-lg-offset-1 container">            
            {% block details %}{% endblock %}
        </div>
    </div>
{% endblock %}

{% block extra_css %}
    <link 
        href="{% static "standingssync/base.css" %}" 
        type="text/css" 
        rel="stylesheet"
    >
{% endblock %}